YUI.add("tweet-search", function (Y) {

    //cache YUI components/set vars
    var Node = Y.Node,
        getClassName = Y.ClassNameManager.getClassName,
        i, j,
        baseClasses = ["_CLASS", "title", "loader", "viewer", "tweet", "ui", "label", "input", "button", "error"],
        templates = [
            "_TEMPLATE",
            "<hgroup class={titleclass}><h1>{title}</h1><h2>{subtitle} <span>{term}</span></h2></hgroup>",
            "<div class={loaderclass}>loading...</div>",
            "<div class={viewerclass}></div>",
            "<article><a href={userurl} title={username}><img src={avatar} alt={username} /><h1>{username}</h1></a><p>{text}</p></article>",
            "<div class={uiclass}></div>",
            "<label class={labelclass}>{labeltext}</label>",
            "<input class={inputclass} />",
            "<button class={buttonclass}>{buttontext}</button>",
            "<p class={errorclass}>{message}</p>"
        ];

    //add widget constructor to YUI
    function TweetSearch(config) {
        TweetSearch.superclass.constructor.apply(this, arguments);
    }

    //namespace the widget
    Y.namespace("DW").TweetSearch = TweetSearch;

    //set name of widget
    TweetSearch.NAME = "tweetsearch";

    //set class names and templates
    for (i = 1, j = baseClasses.length; i < j; i++) {
        var current = baseClasses[i].toUpperCase(),
            prop1 = current + baseClasses[0],
            prop2 = current + templates[0];

        TweetSearch[prop1] = getClassName(TweetSearch.NAME, baseClasses[i]);
        TweetSearch[prop2] = templates[i];
    }

    //set configurable attributes
    TweetSearch.ATTRS = {
        //define attrs and default values
        term: {
            value: "yui3",
            validator: "_validateTerm"
        },
        numberOfTweets: {
            value: 5
        },
        baseURL: {
            value: "http://search.twitter.com/search.json?&with_twitter_user_id=true&include_entities=true&callback={callback}"
        },
        tweets: {
            value: null
        },
        showTitle: {
            value: true
        },
        showUI: {
            value: true
        },

        //text strings used by the widget
        strings: {
            value: {
                title: "Twitter Search Widget",
                subTitle: "Showing results for:",
                label: "Search Term",
                button: "Search",
                errorMsg: "I'm sorry, that search term did not return any results. Please try a different term"
            }
        }
    };

    //use value of input (if present) as term attr
    TweetSearch.HTML_PARSER = {
        term: function (srcNode) {
            var input = srcNode.one("input");

            if (input) {
                var val = input.get("value");
                input.remove();
            }

            return val;
        }
    };

    //Extend the base Widget class
    TweetSearch = Y.extend(TweetSearch, Y.Widget, {

        initializer: function () {
            //get tweets for default/configured term
            this._retrieveTweets();
        },

        renderUI: function () {
            //add widget markup
            var contentBox = this.get("contentBox"),
                strings = this.get("strings"),
                viewer = Node.create(Y.substitute(TweetSearch.VIEWER_TEMPLATE, { viewerclass: TweetSearch.VIEWER_CLASS })),
                loadingNode = Node.create(Y.substitute(TweetSearch.LOADER_TEMPLATE, { loaderclass: TweetSearch.LOADER_CLASS }));

            //insert new markup
            if (this.get("showTitle")) {
                this._createTitle();
            }
            this._loadingNode = contentBox.appendChild(loadingNode);
            this._viewerNode = contentBox.appendChild(viewer);

            //add ui if configured
            if (this.get("showUI")) {
                this._createSearchUI();
            }

            //add widget content class
            contentBox.addClass("yui3-widget-content");
        },

        bindUI: function () {
            if (this.get("showUI")) {
                //perform search when button clicked
                Y.on("click", Y.bind(this._setTerm, this), this._buttonNode);

                //react to ATTR changes
                this.after("termChange", this._afterTermChange);
            }

            this.after("showTitleChange", this._afterShowTitleChange);
            this.after("showUIChange", this._afterShowUIChange);
            this.after("tweetsChange", this._afterTweetsChange);
        },

        //validate term ATTR
        _validateTerm: function (val) {
            return val !== this.get("term");
        },

        //search twitter
        _retrieveTweets: function () {

            //configure request
            var that = this,
                url = [this.get("baseURL"), "&q=", encodeURI(this.get("term")), "&rpp=", this.get("numberOfTweets")].join(""),
                handler = function (data) {
                    //update tweets attribute
                    that.set("tweets", data);
                },
                request = new Y.JSONPRequest(url, handler);

            request.send();
        },

        //create title
        _createTitle: function () {
            var strings = this.get("strings"),
                titleNode = Node.create(Y.substitute(TweetSearch.TITLE_TEMPLATE, { titleclass: TweetSearch.TITLE_CLASS, title: strings.title, subtitle: strings.subTitle, term: this.get("term") }));

            this._titleNode = this.get("contentBox").prepend(titleNode);
        },

        //create the search ui
        _createSearchUI: function () {

            var contentBox = this.get("contentBox"),
                strings = this.get("strings"),
                ui = Node.create(Y.substitute(TweetSearch.UI_TEMPLATE, { uiclass: TweetSearch.UI_CLASS })),
                label = Node.create(Y.substitute(TweetSearch.LABEL_TEMPLATE, { labelclass: TweetSearch.LABEL_CLASS, labeltext: strings.label })),
                input = Node.create(Y.substitute(TweetSearch.INPUT_TEMPLATE, { inputclass: TweetSearch.INPUT_CLASS })),
                button = Node.create(Y.substitute(TweetSearch.BUTTON_TEMPLATE, { buttonclass: TweetSearch.BUTTON_CLASS, buttontext: strings.button }));

            this._uiNode = ui;

            this._labelNode = this._uiNode.appendChild(label);
            this._inputNode = this._uiNode.appendChild(input);
            this._buttonNode = this._uiNode.appendChild(button);

            this._uiNode.appendTo(contentBox);
        },

        //try and update term ATTR
        _setTerm: function () {
            this.set("term", this._inputNode.get("value"));
        },

        //update subtitle
        _uiSetTitle: function (val) {
            this._titleNode.one("h2 span").setContent(val);
        },

    });

}, "0.0.1", { requires: ["widget", "substitute", "jsonp"] });